<?php

require_once dirname(__FILE__).'/../../../SEI.php';

/**
 * Regra de negcio para o parmetros do mdulo PEN
 *
 * @author Join Tecnologia
 */
class PenParametroRN extends InfraRN {

    protected function inicializarObjInfraIBanco(){
        return BancoSEI::getInstance();
    }
    
    protected function contarControlado(PenParametroDTO $objDTO){
        
        try {
            $objBD = new PenParametroBD($this->inicializarObjInfraIBanco());
            return $objBD->contar($objDTO);
        } 
        catch (Exception $e) {
            throw new InfraException('Erro ao contar parmetro.', $e);
        }
        
    }

    protected function consultarControlado(PenParametroDTO $objDTO){
               
        try {
            $objBD = new PenParametroBD($this->inicializarObjInfraIBanco());            
            return $objBD->consultar($objDTO);
        } 
        catch (Exception $e) {
            throw new InfraException('Erro ao listar parmetro.', $e);
        }
    }
    
    protected function listarControlado(PenParametroDTO $objDTO){
               
        try {
            SessaoSEI::getInstance()->validarAuditarPermissao('pen_parametros_configuracao', __METHOD__, $objDTO);
            $objBD = new PenParametroBD($this->inicializarObjInfraIBanco());            
            return $objBD->listar($objDTO);
        } 
        catch (Exception $e) {
            throw new InfraException('Erro ao listar parmetro.', $e);
        }
    }
    
    protected function cadastrarControlado(PenParametroDTO $objDTO){
               
        try {
            $objBD = new PenParametroBD($this->inicializarObjInfraIBanco());            
            return $objBD->cadastrar($objDTO);
        } 
        catch (Exception $e) {
            throw new InfraException('Erro ao cadastrar parmetro.', $e);
        }
    }
    
    protected function alterarControlado(PenParametroDTO $objDTO){
               
        try {
            SessaoSEI::getInstance()->validarAuditarPermissao('pen_parametros_configuracao_alterar', __METHOD__, $objDTO);
            $objBD = new PenParametroBD($this->inicializarObjInfraIBanco());            
            return $objBD->alterar($objDTO);
        } 
        catch (Exception $e) {
            throw new InfraException('Erro ao alterar parmetro.', $e);
        }
    }
    
    protected function excluirControlado(PenParametroDTO $objDTO){
               
        try {
            $objBD = new PenParametroBD($this->inicializarObjInfraIBanco());
            return $objBD->excluir($objDTO);
        } 
        catch (Exception $e) {
            throw new InfraException('Erro ao excluir parmetro.', $e);
        }
    }
        
    public function setValor($strNome, $strValor){
        
        try {
            $objBD = new PenParametroBD($this->inicializarObjInfraIBanco());            
            return $objBD->setValor($strNome, $strValor);
        } 
        catch (Exception $e) {
            throw new InfraException('Erro ao reativar parmetro.', $e);
        }
    }
    
    public function isSetValor($strNome){        
        return $objBD->isSetValor($strNome); 
    }
    
    /**
     * Resgata o valor do parmetro
     * @param string $strNome
     */
    public function getParametro($strNome) {
        $objPenParametroDTO = new PenParametroDTO();
        $objPenParametroDTO->setStrNome($strNome);
        $objPenParametroDTO->retStrValor();

        if($this->contar($objPenParametroDTO) > 0) {
            $objPenParametroDTO = $this->consultar($objPenParametroDTO);
            return $objPenParametroDTO->getStrValor();
        }
    }
}
